SubmapLayout {
    stroke: none;
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
}

CompartmentLayout {
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
}

GenericProcessLayout {
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
}

UncertainProcessLayout {
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
}

OmittedProcessLayout {
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
}

AssociationLayout {
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
}

DissociationLayout {
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
}

PhenotypeLayout {
    stroke: none;
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
}

AndOperatorLayout {
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
}

OrOperatorLayout {
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
}

NotOperatorLayout {
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
}

EquivalenceOperatorLayout {
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
}

UnspecifiedEntityLayout {
    stroke: none;
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
}

MacromoleculeLayout {
    stroke: none;
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
}

MacromoleculeMultimerLayout {
    stroke: none;
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
    subunits-group-filter: none, drop-shadow(1.0, 1.0, 2.0, 0.5, black);
}

ComplexLayout {
    stroke: none;
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
}

ComplexMultimerLayout {
    stroke: none;
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
    subunits-group-filter: none, drop-shadow(1.0, 1.0, 2.0, 0.5, black);
}

SimpleChemicalLayout {
    stroke: none;
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
}

SimpleChemicalMultimerLayout {
    stroke: none;
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
    subunits-group-filter: none, drop-shadow(1.0, 1.0, 2.0, 0.5, black);
}

NucleicAcidFeatureLayout {
    stroke: none;
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
}

NucleicAcidFeatureMultimerLayout {
    stroke: none;
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
    subunits-group-filter: none, drop-shadow(1.0, 1.0, 2.0, 0.5, black);
}

EmptySetLayout {
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
}

PerturbingAgentLayout {
    stroke: none;
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
}

StateVariableLayout {
    stroke: none;
}

UnitOfInformationLayout {
    stroke: none;
}

TagLayout {
    stroke: none;
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
}

SubmapTerminalLayout {
    stroke: none;
}

MacromoleculeUnitOfInformationLayout {
    stroke: none;
}

UnspecifiedEntityUnitOfInformationLayout {
    stroke: none;
}

NucleicAcidFeatureUnitOfInformationLayout {
    stroke: none;
}

ComplexUnitOfInformationLayout {
    stroke: none;
}

SimpleChemicalUnitOfInformationLayout {
    stroke: none;
}

PerturbationUnitOfInformationLayout {
    stroke: none;
}

DelayOperatorLayout {
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
}

BiologicalActivityLayout {
    stroke: none;
    group-filter: drop-shadow(3.0, 3.0, 2.0, 0.5, black);
}
