AndOperatorLayout {
	fill: white;
	stroke: black;
	group-fill: unset;
	left-connector-fill: unset;
	left-connector-stroke: unset;
	right-connector-fill: unset;
	right-connector-stroke: unset;
	group-stroke: unset;
}


AssociationLayout {
	fill: black;
	stroke: black;
	group-fill: unset;
	left-connector-fill: unset;
	left-connector-stroke: unset;
	right-connector-fill: unset;
	right-connector-stroke: unset;
	group-stroke: unset;
}


BiologicalActivityLayout {
	fill: blue;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


CardinalityLayout {
	fill: white;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


CatalysisLayout {
	arrowhead-fill: white;
	arrowhead-stroke: black;
	group-fill: unset;
	path-fill: none;
	path-stroke: black;
	group-stroke: unset;
}


CompartmentLayout {
	fill: ghostwhite;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


ComplexLayout {
	fill: gold;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


ComplexMultimerLayout {
	fill: gold;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
	subunits-fill: unset;
	subunits-stroke: unset;
}


ComplexMultimerSubunitLayout {
	fill: gold;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
	subunits-fill: unset;
	subunits-stroke: unset;
}


ComplexSubunitLayout {
	fill: gold;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


ComplexUnitOfInformationLayout {
	fill: gold;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


ConsumptionLayout {
	arrowhead-fill: white;
	arrowhead-stroke: black;
	group-fill: unset;
	path-fill: none;
	path-stroke: black;
	group-stroke: unset;
}


DelayOperatorLayout {
	fill: white;
	stroke: black;
	group-fill: unset;
	left-connector-fill: unset;
	left-connector-stroke: unset;
	right-connector-fill: unset;
	right-connector-stroke: unset;
	group-stroke: unset;
}


DissociationLayout {
	fill: white;
	stroke: black;
	group-fill: unset;
	left-connector-fill: unset;
	left-connector-stroke: unset;
	right-connector-fill: unset;
	right-connector-stroke: unset;
	group-stroke: unset;
}


EmptySetLayout {
	fill: white;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


EquivalenceArc {
	arrowhead-fill: unset;
	arrowhead-stroke: unset;
	group-fill: unset;
	path-fill: unset;
	path-stroke: unset;
	group-stroke: unset;
}


EquivalenceOperatorLayout {
	fill: white;
	stroke: black;
	group-fill: unset;
	left-connector-fill: unset;
	left-connector-stroke: unset;
	right-connector-fill: unset;
	right-connector-stroke: unset;
	group-stroke: unset;
}


GenericProcessLayout {
	fill: white;
	stroke: black;
	group-fill: unset;
	left-connector-fill: unset;
	left-connector-stroke: unset;
	right-connector-fill: unset;
	right-connector-stroke: unset;
	group-stroke: unset;
}


InhibitionLayout {
	arrowhead-fill: white;
	arrowhead-stroke: black;
	group-fill: unset;
	path-fill: none;
	path-stroke: black;
	group-stroke: unset;
}


LogicArc {
	arrowhead-fill: unset;
	arrowhead-stroke: unset;
	group-fill: unset;
	path-fill: unset;
	path-stroke: unset;
	group-stroke: unset;
}


MacromoleculeLayout {
	fill: deepskyblue;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


MacromoleculeMultimerLayout {
	fill: deepskyblue;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
	subunits-fill: unset;
	subunits-stroke: unset;
}


MacromoleculeMultimerSubunitLayout {
	fill: deepskyblue;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
	subunits-fill: unset;
	subunits-stroke: unset;
}


MacromoleculeSubunitLayout {
	fill: deepskyblue;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


MacromoleculeUnitOfInformationLayout {
	fill: deepskyblue;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


ModulationLayout {
	arrowhead-fill: white;
	arrowhead-stroke: black;
	group-fill: unset;
	path-fill: none;
	path-stroke: black;
	group-stroke: unset;
}


NecessaryStimulationLayout {
	arrowhead-fill: white;
	arrowhead-stroke: black;
	group-fill: unset;
	path-fill: none;
	path-stroke: black;
	group-stroke: unset;
}


NegativeInfluenceLayout {
	arrowhead-fill: white;
	arrowhead-stroke: black;
	group-fill: unset;
	path-fill: none;
	path-stroke: black;
	group-stroke: unset;
}


NotOperatorLayout {
	fill: white;
	stroke: black;
	group-fill: unset;
	left-connector-fill: unset;
	left-connector-stroke: unset;
	right-connector-fill: unset;
	right-connector-stroke: unset;
	group-stroke: unset;
}


NucleicAcidFeatureLayout {
	fill: limegreen;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


NucleicAcidFeatureMultimerLayout {
	fill: limegreen;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
	subunits-fill: unset;
	subunits-stroke: unset;
}


NucleicAcidFeatureMultimerSubunitLayout {
	fill: limegreen;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
	subunits-fill: unset;
	subunits-stroke: unset;
}


NucleicAcidFeatureSubunitLayout {
	fill: limegreen;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


NucleicAcidFeatureUnitOfInformationLayout {
	fill: limegreen;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


OmittedProcessLayout {
	fill: white;
	stroke: black;
	group-fill: unset;
	left-connector-fill: unset;
	left-connector-stroke: unset;
	right-connector-fill: unset;
	right-connector-stroke: unset;
	group-stroke: unset;
}


OrOperatorLayout {
	fill: white;
	stroke: black;
	group-fill: unset;
	left-connector-fill: unset;
	left-connector-stroke: unset;
	right-connector-fill: unset;
	right-connector-stroke: unset;
	group-stroke: unset;
}


PerturbationUnitOfInformationLayout {
	fill: orangered;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


PerturbingAgentLayout {
	fill: orangered;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


PhenotypeLayout {
	fill: plum;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


PositiveInfluenceLayout {
	arrowhead-fill: white;
	arrowhead-stroke: black;
	group-fill: unset;
	path-fill: none;
	path-stroke: black;
	group-stroke: unset;
}


ProductionLayout {
	arrowhead-fill: black;
	arrowhead-stroke: black;
	group-fill: unset;
	path-fill: none;
	path-stroke: black;
	group-stroke: unset;
}


SBGNAFLayout {
	fill: white;
	stroke: unset;
	group-fill: unset;
	group-stroke: unset;
}


SBGNPDLayout {
	fill: white;
	stroke: unset;
	group-fill: unset;
	group-stroke: unset;
}


SimpleChemicalLayout {
	fill: sandybrown;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


SimpleChemicalMultimerLayout {
	fill: sandybrown;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
	subunits-fill: unset;
	subunits-stroke: unset;
}


SimpleChemicalMultimerSubunitLayout {
	fill: sandybrown;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
	subunits-fill: unset;
	subunits-stroke: unset;
}


SimpleChemicalSubunitLayout {
	fill: sandybrown;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


SimpleChemicalUnitOfInformationLayout {
	fill: sandybrown;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


StateVariableLayout {
	fill: white;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


StimulationLayout {
	arrowhead-fill: unset;
	arrowhead-stroke: unset;
	group-fill: unset;
	path-fill: unset;
	path-stroke: unset;
	group-stroke: unset;
}


SubmapLayout {
	fill: tan;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


TagLayout {
	fill: white;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


TerminalLayout {
	fill: white;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


UncertainProcessLayout {
	fill: white;
	stroke: black;
	group-fill: unset;
	left-connector-fill: unset;
	left-connector-stroke: unset;
	right-connector-fill: unset;
	right-connector-stroke: unset;
	group-stroke: unset;
}


UnitOfInformationLayout {
	fill: white;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


UnknownInfluenceLayout {
	arrowhead-fill: white;
	arrowhead-stroke: black;
	group-fill: unset;
	path-fill: none;
	path-stroke: black;
	group-stroke: unset;
}


UnspecifiedEntityLayout {
	fill: lightgray;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


UnspecifiedEntitySubunitLayout {
	fill: lightgray;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}


UnspecifiedEntityUnitOfInformationLayout {
	fill: lightgray;
	stroke: black;
	group-fill: unset;
	group-stroke: unset;
}
